@@ -1,12 +1,13 @@
-# Downloader / extracter for latest iLO2 / iLO3 / iLO4 firmware
+# Downloader / extracter for latest iLO2 / iLO3 / iLO4 / iLO5 firmware
 #
-# (c) 2011-2016 Dennis Kaarsemaker <dennis@kaarsemaker.net>
+# (c) 2011-2020 Dennis Kaarsemaker <dennis@kaarsemaker.net>
 # see COPYING for license details
 
 import tarfile
 import io
 import os
 import sys
+from zipfile import ZipFile
 PY3 = sys.version_info[0] >= 3
 
 if PY3:
@@ -48,8 +49,11 @@ def download(ilo, path=None, progress = lambda txt: None):
         progress(msg)
         data = _download(conf[ilo]['url'], lambda txt: progress('%s %s' % (msg, txt)))
         if conf[ilo]['url'].endswith('.bin'):
-            with open(os.path.join(path, conf[ilo]['file']), 'w') as fd:
+            with open(os.path.join(path, conf[ilo]['file']), 'wb') as fd:
                 fd.write(data)
+        elif conf[ilo]['url'].endswith('.fwpkg'):
+            with ZipFile(io.BytesIO(data)) as zipObj:
+                zipObj.extract(conf[ilo]['file'], path)
         else:
             _parse(data, path, conf[ilo]['file'])
         return True
